#!/bin/bash

# Get variables from settings.sh
source settings.sh

ZIP_PASSWORD=`cat $ZIP_PASSWORD_FILE`
ENCRYPTION_KEY=`cat $ENCRYPTION_KEY_FILE`
ENCRYPTION_IV=`cat $ENCRYPTION_IV_FILE`

SYSTEM_IMAGE_PATH=$UPDATE_EXTRACT_TEMP_DIR/enc_system/system.img
UPDATE_LIST_PATH=$UPDATE_EXTRACT_TEMP_DIR/update.cfg
INFO_SIGN_FILE=$UPDATE_EXTRACT_TEMP_DIR/update.info
TEMP_HASH_PATH=sha.txt

function create_update_list() {
	if [[ ! -f $UPDATE_LIST_PATH ]];then
		echo "Could not find update list file in ${UPDATE_EXTRACT_TEMP_DIR}"
		exit
	fi
	
	echo "Calculating new system.img file hash"
	generate_sha224_hash $SYSTEM_IMAGE_PATH
	
	sed -i "s/system\/system.img:.*/system\/system.img:${SHA224_STR}/g" $UPDATE_LIST_PATH
	
	echo "Updated update list file hashes"
}

function sign_update_list() {		
	echo "Calculating update.cfg hash"
	
	generate_sha224_hash ${UPDATE_LIST_PATH}			
	echo "${SHA224_STR}" > "${TEMP_HASH_PATH}"
	
	rm $INFO_SIGN_FILE
	echo "Cleared existing signing file"
	
	echo "Signing update.cfg"
	RSA_STR=`openssl rsautl -sign -in ${TEMP_HASH_PATH} -inkey ${RSA_PRIVATE_SIGNING_KEY_FILE} -out ${INFO_SIGN_FILE}`

	`rm ${TEMP_HASH_PATH}`
}

function generate_zip() {
	local ENC_UPDATE_PACKAGE="enc_system_package_PUT_VERSION_HERE.zip"
	
	cd $UPDATE_EXTRACT_TEMP_DIR
	zip -P $ZIP_PASSWORD -r ../$ENC_UPDATE_PACKAGE .
	
	cd ../
	
	echo "Update file created: $ENC_UPDATE_PACKAGE"
}

function generate_sha224_hash() {
	local FILE_PATH=$1
  
	local SHA_BUF_SIZE=16777088
  
	echo "FILE=${FILE_PATH}"
  
	fsize=$(stat -c%s ${FILE_PATH})
	
	i=0
	cnt=`expr $fsize / $SHA_BUF_SIZE`
	while [ "$i" -le "$cnt" ]
	do 
		dd if=${FILE_PATH} of=${FILE_PATH}_$i bs=$SHA_BUF_SIZE count=1 skip=$i;

	  	TMP=$(sha224sum ${FILE_PATH}_$i|awk '{print$1}');

		if [ ! -r "./subHash" ]; then
			echo -ne $TMP > ./subHash;
		else
			echo -ne $TMP >> ./subHash;			
		fi

	  	rm -rf ${FILE_PATH}_$i
	
	  	i=$((i+1));
	done

	xxd -r -p ./subHash > ./subBinary
	od -v -t x1 ./subBinary
  	SHA224_STR=`openssl dgst -sha224 subBinary`

	SHA224_STR=${SHA224_STR:`expr index "${SHA224_STR}" \ `}

	echo $SHA224_STR
	rm -rf ./subHash ./subBinary
}

echo "Syncing to save changes to system.img mount"
sync

echo "Unmounting system.img"
sudo umount $SYSTEM_IMAGE_MOUNT_DIR

echo "Calculating file list"
create_update_list

echo "Signing update list"
sign_update_list

echo "Generating Update zip"
generate_zip